"""Facet definitions."""

from invenio_records_resources.services.records.facets import TermsFacet
from invenio_search.engine import dsl
from oarepo_runtime.facets.nested_facet import NestedLabeledFacet


{% for dict in search_options_data %}
{% for key, value in dict.items()%}
{{ key }} = {{ value }}
{% endfor %}
{% endfor %}

{{ current_model.invenio_record_facets_extra_code }}